import java.net.Socket;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.IOException;

/**
 * Ein einfacher Clientantwortprozess fr den Spielserver.
 * 
 * @author Q11 
 * @version 2015-07-01
 */
public class ClientAntwortProzess implements Runnable
{
    private Socket meinSocket;
    private ServerProzess server;
    private String meinNick;
    private int meinRot;
    private int meinGruen;
    private int meinBlau;
    private Serverprotokoll protokoll;

    private PrintWriter gibAnClient;
    private BufferedReader liesVomClient;

    /**
     * Konstruktor.
     * Erzeuge einen neuen Clientprozess am Socket s fr
     * den Serverprozess p.
     */
    public ClientAntwortProzess (Socket s, ServerProzess p)
    {
        meinSocket = s;
        server     = p;
        meinNick   = null;
        protokoll  = null;
    }

    /**
     * Ablaufplan.
     */
    public void run()
    {
        try
        {
            /*
             * Datenstrme an den und vom Client.
             */
            gibAnClient   = new PrintWriter (meinSocket.getOutputStream (), true);
            liesVomClient = new BufferedReader (new InputStreamReader (meinSocket.getInputStream ()));

            /*
             * Zustndiges Protokoll.
             */
            protokoll = new Serverprotokoll ();

            /*
             * Ein- und Ausgabe und der Serverbefehl.
             */
            String eingabeZeile;
            Antwort ausgabe;
            String befehl;

            /*
             * Neuer Nick beim Nickwechsel.
             */
            String neuerNick = null;

            /*
             * Aktivitt des Clients.
             */
            boolean aktiv = true;

            /*
             * Reagiere zunchst auf die leere Eingabe vom Client.
             */
            ausgabe = protokoll.verarbeite (null);
            sende (ausgabe.gibNachrichtAnMich ());

            /*
             * Chatte gem Protokoll, solange der Client sendet.
             */
            eingabeZeile = liesVomClient.readLine ();

            if (eingabeZeile == null)
            {
                aktiv = false;
            }

            while (aktiv)
            {
                ausgabe = protokoll.verarbeite (eingabeZeile);
                befehl  = ausgabe.gibBefehl ();

                /*
                 * Wenn eine Nachricht an eine einzelne Adressatin vorliegt,
                 * sende diese.
                 */
                if (null != ausgabe.gibAdressatin () && null != ausgabe.gibNachrichtAnAdressatin ())
                {
                    server.sendeAn (ausgabe.gibAdressatin (), ausgabe.gibNachrichtAnAdressatin (), this);
                }

                /*
                 * Wenn eine Nachricht an alle vorliegt, sende diese.
                 */
                if (null != ausgabe.gibNachrichtAnAlle ())
                {
                    server.sendeAnAlle (ausgabe.gibNachrichtAnAlle ());
                }

                /*
                 * Wenn eine Nachricht an mich selbst vorliegt, sende diese.
                 */
                if (null != ausgabe.gibNachrichtAnMich ())
                {
                    sende (ausgabe.gibNachrichtAnMich ());
                }

                /*
                 * Wenn ein Befehl vorliegt, fhre ihn aus.
                 */
                if (null != befehl)
                {
                    if (befehl.equals ("quit"))
                    {
                        sende (ausgabe.gibNachrichtAnMich ());
                        aktiv = false;
                        break;
                    }
                    else if (befehl.equals ("nick"))
                    {
                        neuerNick = ausgabe.gibArgument ();

                        if (server.pruefeNick (neuerNick))
                        {
                            sende ("Enter password:");
                            protokoll.gibPasswort ();
                        }
                        else
                        {
                            boolean nickAenderungErfolgreich = server.aendereNick (meinNick, neuerNick, this);

                            if (nickAenderungErfolgreich)
                            {
                                meinNick = neuerNick;
                                protokoll.setzeNick (meinNick);
                                sende ("Name: " + meinNick);
                                sende ("Choose password:");
                                protokoll.waehlePasswort ();
                            }
                        }
                    }
                    else if (befehl.equals ("passwort"))
                    {
                        server.setzePasswort (meinNick, ausgabe.gibArgument ());
                        protokoll.gibRot ();
                        sende ("Rot eingeben!");
                    }
                    else if (befehl.equals ("authentication"))
                    {
                        if (server.pruefePasswort (neuerNick, ausgabe.gibArgument ()))
                        {
                            server.aendereNick (meinNick, neuerNick, this);

                            meinNick = neuerNick;
                            protokoll.chatte ();
                            protokoll.setzeNick (meinNick);
                            sende ("Name: " + meinNick);
                            server.nenneFarbe (meinNick);
                        }
                        else
                        {  
                            sende ("Wrong password.");
                            sende ("Please try again or 'quit'.");
                            sende ("Enter password:");
                        }
                    }
                    else if (befehl.equals ("color"))
                    { 
                        protokoll.gibRot ();
                        sende ("Rot eingeben!");
                    }
                    else if (befehl.equals ("rot"))
                    {
                        int neueZahl = -1;

                        try
                        {
                            neueZahl = Integer.parseInt (ausgabe.gibArgument ());
                        }
                        catch (Exception fehler)
                        {
                        }

                        if (neueZahl >= 0 && neueZahl <= 255)
                        {
                            meinRot = neueZahl; 
                            protokoll.gibGruen();
                            sende ("Gruen eingeben!");
                        }
                        else
                        {
                            sende ("Falscher Farbwert! :( Die Zahl muss zwischen 0 und 255 liegen!");
                        }
                    }
                    else if (befehl.equals ("gruen"))
                    {
                        int neueZahl = -1;

                        try
                        {
                            neueZahl = Integer.parseInt (ausgabe.gibArgument ());
                        }
                        catch (Exception fehler)
                        {
                        }

                        if (neueZahl >= 0 && neueZahl <= 255)
                        {
                            meinGruen= neueZahl; 
                            protokoll.gibBlau();
                            sende ("Blau eingeben!");
                        }
                        else
                        {
                            sende("Falscher Farbwert! :( Die Zahl muss zwischen 0 und 255 liegen!");
                        }
                    }
                    else if (befehl.equals ("blau"))
                    {
                        int neueZahl = -1;

                        try
                        {
                            neueZahl = Integer.parseInt (ausgabe.gibArgument ());
                        }
                        catch (Exception fehler)
                        {
                        }

                        if (neueZahl >= 0 && neueZahl <= 255)
                        {
                            meinBlau= neueZahl;

                            if (server.aendereFarbe (meinNick, meinRot, meinGruen, meinBlau))
                            {
                                protokoll.chatte();
                                server.nenneFarbe (meinNick);
                            }
                            else
                            {
                                protokoll.gibRot ();
                                sende ("Rot eingeben!");
                            }
                        }
                        else
                        {
                            sende ("Falscher Farbwert! :( Die Zahl muss zwischen 0 und 255 liegen!");
                        }
                    }
                    else if (befehl.equals ("who"))
                    {
                        server.nenneNicks (meinNick, ausgabe.gibArgument ());
                    }
                    else if (befehl.equals ("play"))
                    {
                        boolean korrekt = true;

                        int groesseX;
                        int groesseY;

                        try
                        {
                            groesseX = Integer.parseInt (ausgabe.gibZweitesArgument ());
                        }
                        catch (Exception fehler)
                        {
                            sende ("Invalid width " + ausgabe.gibZweitesArgument () + ".");
                            groesseX = 1;
                            korrekt = false;
                        }

                        if (groesseX < 1)
                        {
                            sende ("Width " + groesseX + " too small (minimum 1).");
                            korrekt = false;
                        }

                        if (groesseX > 60)
                        {
                            sende ("Width " + groesseX + " too large (maximum 60).");
                            korrekt = false;
                        }   

                        try
                        {
                            groesseY = Integer.parseInt (ausgabe.gibDrittesArgument ());
                        }
                        catch (Exception fehler)
                        {
                            sende ("Invalid height " + ausgabe.gibZweitesArgument () + ".");
                            groesseY = 1;
                            korrekt = false;
                        }

                        if (groesseY < 1)
                        {
                            sende ("Height " + groesseY + " too small (minimum 1).");
                            korrekt = false;
                        }

                        if (groesseY > 35)
                        {
                            sende ("Height " + groesseY + " too large (maximum 35).");
                            korrekt = false;
                        }

                        if (korrekt)
                        {
                            server.wuenscheSpiel (meinNick, ausgabe.gibArgument(), groesseX, groesseY);
                        }
                    }
                    else if (befehl.equals ("move quer"))
                    {
                        server.spiele (meinNick, ausgabe.gibArgument (), ausgabe.gibZweitesArgument (), "quer");
                    }
                    else if (befehl.equals ("move hoch"))
                    {
                        server.spiele (meinNick, ausgabe.gibArgument (), ausgabe.gibZweitesArgument (), "hoch");
                    }
                    else if (befehl.equals ("resign"))
                    {
                        server.gibAuf (meinNick);
                    }
                    else if (befehl.equals ("done"))
                    {
                        server.beendeSpiel (meinNick);
                    }
                    else if (befehl.equals ("points"))
                    {
                        server.nennePunkte (meinNick, ausgabe.gibArgument ());
                    }
                    else if (befehl.equals ("new"))
                    {
                        neuerNick = ausgabe.gibArgument ();

                        if (server.pruefeNick (neuerNick))
                        {
                            sende ("Enter password:");
                            protokoll.gibPasswort ();
                        }
                        else
                        {
                            boolean nickAenderungErfolgreich = server.aendereNick (meinNick, neuerNick, this);

                            if (nickAenderungErfolgreich)
                            {
                                meinNick = neuerNick;
                                protokoll.chatte ();
                                protokoll.setzeNick (meinNick);
                                sende ("Name: " + meinNick);
                            }
                        }
                    }
                }

                eingabeZeile = liesVomClient.readLine ();

                if (eingabeZeile == null)
                {
                    aktiv = false;
                }
            }

            /*
             * Melde Dich vom Server ab.
             */
            server.meldeAb (meinNick);

            /*
             * Schliee die Datenstrme und den Socket.
             */
            gibAnClient.close ();
            liesVomClient.close ();
            meinSocket.close();
        }
        catch (IOException fehler)
        {
            fehler.printStackTrace();
        }
    }

    /**
     * Sende die angegebene Nachricht an den Client.
     */
    public synchronized boolean sende (String nachricht)
    {
        if (gibAnClient != null)
        {
            gibAnClient.println (nachricht);

            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Nenne meine Adresse im Netz.
     */
    public String gibAdresse ()
    {
        return meinSocket.getInetAddress ().getCanonicalHostName () + " ("
        + meinSocket.getInetAddress ().getHostAddress () + ")";
    }

    /**
     * Chatte.
     */
    public void chatte ()
    {
        protokoll.chatte ();
    }

    /**
     * Spiele ein Spiel gegen die angegebene Gegnerin.
     */
    public void spiele (String gegnerin)
    {
        protokoll.spiele (gegnerin);
    }

    /**
     * Werte ein beendetes Spiel.
     */
    public void werte ()
    {
        protokoll.werte ();
    }
}